@@ -1,7 +1,6 @@
 
 /* Compiler implementation of the D programming language
- * Copyright (c) 1999-2016 by The D Language Foundation
- * All Rights Reserved
+ * Copyright (C) 1999-2018 by The D Language Foundation, All Rights Reserved
  * written by Walter Bright
  * http://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
@@ -22,5 +21,9 @@ struct OutBuffer;
 
 void json_generate(OutBuffer *, Modules *);
 
+#ifdef IN_LLVM
+unsigned tryParseJsonField(const char *fieldName);
+#endif
+
 #endif /* DMD_JSON_H */
 
