@@ -285,22 +285,22 @@ struct Array
         return *this;
     }
 
-    size_type size()
+    size_type size() const
     {
         return static_cast<size_type>(dim);
     }
 
-    bool empty()
+    bool empty() const
     {
         return dim == 0;
     }
 
-    TYPE front()
+    TYPE front() const
     {
         return data[0];
     }
 
-    TYPE back()
+    TYPE back() const
     {
         return data[dim-1];
     }
@@ -316,27 +316,19 @@ struct Array
     }
 
     typedef TYPE *iterator;
+    typedef const TYPE *const_iterator;
     typedef std::reverse_iterator<iterator> reverse_iterator;
+    typedef std::reverse_iterator<const_iterator> const_reverse_iterator;
 
-    iterator begin()
-    {
-        return static_cast<iterator>(&data[0]);
-    }
+    iterator begin() { return static_cast<iterator>(&data[0]); }
+    iterator end() { return static_cast<iterator>(&data[dim]); }
+    reverse_iterator rbegin() { return reverse_iterator(end()); }
+    reverse_iterator rend() { return reverse_iterator(begin()); }
 
-    iterator end()
-    {
-        return static_cast<iterator>(&data[dim]);
-    }
-
-    reverse_iterator rbegin()
-    {
-        return reverse_iterator(end());
-    }
-
-    reverse_iterator rend()
-    {
-        return reverse_iterator(begin());
-    }
+    const_iterator begin() const { return static_cast<const_iterator>(&data[0]); }
+    const_iterator end() const { return static_cast<const_iterator>(&data[dim]); }
+    const_reverse_iterator rbegin() const { return const_reverse_iterator(end()); }
+    const_reverse_iterator rend() const { return const_reverse_iterator(begin()); }
 
     iterator erase(iterator pos)
     {
