@@ -45,11 +45,6 @@ struct Port
     static unsigned readlongBE(void *buffer);
     static unsigned readwordLE(void *buffer);
     static unsigned readwordBE(void *buffer);
-
-#ifdef IN_LLVM
-    static int stricmp(const char *s1, const char *s2);
-#endif
-
     static void valcpy(void *dst, uint64_t val, size_t size);
 };
 
