@@ -517,6 +517,7 @@ public:
 class SwitchErrorStatement : public Statement
 {
 public:
+    Expression *exp;
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -724,6 +725,14 @@ class AsmStatement : public Statement
 {
 public:
     Token *tokens;
+
+    Statement *syntaxCopy();
+    void accept(Visitor *v) { v->visit(this); }
+};
+
+class InlineAsmStatement : public AsmStatement
+{
+public:
     code *asmcode;
     unsigned asmalign;          // alignment of this statement
     unsigned regs;              // mask of registers modified (must match regm_t in back end)
@@ -733,10 +742,29 @@ public:
 #if IN_LLVM
     // non-zero if this is a branch, contains the target label
     LabelDsymbol* isBranchToLabel;
+
+    static InlineAsmStatement *create(const Loc &loc, Token *tokens);
 #endif
 
     Statement *syntaxCopy();
+    void accept(Visitor *v) { v->visit(this); }
+};
 
+// A GCC asm statement - assembler instructions with D expression operands
+class GccAsmStatement : public AsmStatement
+{
+public:
+    StorageClass stc;           // attributes of the asm {} block
+    Expression *insn;           // string expression that is the template for assembler code
+    Expressions *args;          // input and output operands of the statement
+    unsigned outputargs;        // of the operands in 'args', the number of output operands
+    Identifiers *names;         // list of symbolic names for the operands
+    Expressions *constraints;   // list of string constants specifying constraints on operands
+    Expressions *clobbers;      // list of string constants specifying clobbers and scratch registers
+    Identifiers *labels;        // list of goto labels
+    GotoStatements *gotos;      // of the goto labels, the equivalent statements they represent
+
+    Statement *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
