@@ -44,8 +44,6 @@ struct Target
     // C++ ABI
     static bool reverseCppOverloads;    // with dmc and cl, overloaded functions are grouped and in reverse order
     static bool cppExceptions;          // set if catching C++ exceptions is supported
-    static char int64Mangle;            // mangling character for C++ int64_t
-    static char uint64Mangle;           // mangling character for C++ uint64_t
     static bool twoDtorInVtable;        // target C++ ABI puts deleting and non-deleting destructor into vtable
 
     template <typename T>
