@@ -312,7 +312,13 @@ void outputIR2ObjRelevantCmdlineArgs(llvm::raw_ostream &hash_os) {
 #else
   hash_os << opts::getRelocModel();
 #endif
+#if LDC_LLVM_VER >= 600
+  const auto codeModel = opts::getCodeModel();
+  if (codeModel.hasValue())
+    hash_os << codeModel.getValue();
+#else
   hash_os << opts::getCodeModel();
+#endif
   hash_os << opts::disableFPElim();
 }
 
@@ -466,6 +472,9 @@ void recoverObjectFile(llvm::StringRef cacheObjectHash,
   {
     int FD;
     if (llvm::sys::fs::openFileForWrite(cacheFile.c_str(), FD,
+#if LDC_LLVM_VER >= 700
+                                        llvm::sys::fs::CD_OpenExisting,
+#endif
                                         llvm::sys::fs::F_Append)) {
       error(Loc(), "Failed to open the cached file for writing: %s",
             cacheFile.c_str());
