@@ -7,9 +7,9 @@
 //
 //===----------------------------------------------------------------------===//
 
+#include "driver/linker.h"
 #include "errors.h"
 #include "driver/cl_options.h"
-#include "driver/linker.h"
 #include "driver/tool.h"
 #include "gen/llvm.h"
 #include "gen/logger.h"
@@ -17,31 +17,65 @@
 #include "llvm/Linker/Linker.h"
 #include "llvm/Support/FileSystem.h"
 #include "llvm/Support/SourceMgr.h"
+#include <sstream>
+
+namespace cl = llvm::cl;
 
 //////////////////////////////////////////////////////////////////////////////
 
-static llvm::cl::opt<llvm::cl::boolOrDefault>
-    staticFlag("static", llvm::cl::ZeroOrMore,
-               llvm::cl::desc("Create a statically linked binary, including "
-                              "all system dependencies"));
-
 #if LDC_WITH_LLD
-static llvm::cl::opt<bool>
-    useInternalLinker("link-internally", llvm::cl::ZeroOrMore, llvm::cl::Hidden,
-                      llvm::cl::desc("Use internal LLD for linking"));
+static cl::opt<bool> linkInternally("link-internally", cl::ZeroOrMore,
+                                    cl::desc("Use internal LLD for linking"),
+                                    cl::cat(opts::linkingCategory));
 #else
-constexpr bool useInternalLinker = false;
+constexpr bool linkInternally = false;
 #endif
 
+static cl::opt<bool> noDefaultLib(
+    "nodefaultlib", cl::ZeroOrMore, cl::Hidden,
+    cl::desc("Don't add a default library for linking implicitly"));
+
+static cl::opt<std::string>
+    defaultLib("defaultlib", cl::ZeroOrMore, cl::value_desc("lib1,lib2,..."),
+               cl::desc("Default libraries to link with (overrides previous)"),
+               cl::cat(opts::linkingCategory));
+
+static cl::opt<std::string> debugLib(
+    "debuglib", cl::ZeroOrMore, cl::Hidden, cl::value_desc("lib1,lib2,..."),
+    cl::desc("Debug versions of default libraries (overrides previous). If the "
+             "option is omitted, LDC will append -debug to the -defaultlib "
+             "names when linking with -link-defaultlib-debug"),
+    cl::cat(opts::linkingCategory));
+
+static cl::opt<bool> linkDefaultLibDebug(
+    "link-defaultlib-debug", cl::ZeroOrMore,
+    cl::desc("Link with debug versions of default libraries"),
+    cl::cat(opts::linkingCategory));
+static cl::alias _linkDebugLib("link-debuglib", cl::Hidden,
+                               cl::aliasopt(linkDefaultLibDebug),
+                               cl::desc("Alias for -link-defaultlib-debug"),
+                               cl::cat(opts::linkingCategory));
+
+static cl::opt<cl::boolOrDefault> linkDefaultLibShared(
+    "link-defaultlib-shared", cl::ZeroOrMore,
+    cl::desc("Link with shared versions of default libraries"),
+    cl::cat(opts::linkingCategory));
+
+static cl::opt<cl::boolOrDefault>
+    staticFlag("static", cl::ZeroOrMore,
+               cl::desc("Create a statically linked binary, including "
+                        "all system dependencies"),
+               cl::cat(opts::linkingCategory));
+
 //////////////////////////////////////////////////////////////////////////////
 
 // linker-gcc.cpp
-int linkObjToBinaryGcc(llvm::StringRef outputPath, bool useInternalLinker,
-                       llvm::cl::boolOrDefault fullyStaticFlag);
+int linkObjToBinaryGcc(llvm::StringRef outputPath,
+                       const std::vector<std::string> &defaultLibNames);
 
 // linker-msvc.cpp
-int linkObjToBinaryMSVC(llvm::StringRef outputPath, bool useInternalLinker,
-                        llvm::cl::boolOrDefault fullyStaticFlag);
+int linkObjToBinaryMSVC(llvm::StringRef outputPath,
+                        const std::vector<std::string> &defaultLibNames);
 
 //////////////////////////////////////////////////////////////////////////////
 
@@ -54,6 +88,9 @@ static std::string getOutputName() {
     extension = global.dll_ext;
   } else if (triple.isOSWindows()) {
     extension = "exe";
+  } else if (triple.getArch() == llvm::Triple::wasm32 ||
+             triple.getArch() == llvm::Triple::wasm64) {
+    extension = "wasm";
   }
 
   if (global.params.exefile) {
@@ -64,9 +101,10 @@ static std::string getOutputName() {
   }
 
   // Infer output name from first object file.
-  std::string result = global.params.objfiles->dim
-                           ? FileName::removeExt((*global.params.objfiles)[0])
-                           : "a.out";
+  std::string result =
+      global.params.objfiles.dim
+          ? FileName::removeExt(FileName::name(global.params.objfiles[0]))
+          : "a.out";
 
   if (sharedLib && !triple.isWindowsMSVCEnvironment())
     result = "lib" + result;
@@ -76,9 +114,8 @@ static std::string getOutputName() {
     // after execution. Make sure the name does not collide with other files
     // from other processes by creating a unique filename.
     llvm::SmallString<128> tempFilename;
-    auto EC = llvm::sys::fs::createTemporaryFile(FileName::name(result.c_str()),
-                                                 extension ? extension : "",
-                                                 tempFilename);
+    auto EC = llvm::sys::fs::createTemporaryFile(
+        result, extension ? extension : "", tempFilename);
     if (!EC)
       result = tempFilename.str();
   } else if (extension) {
@@ -91,6 +128,53 @@ static std::string getOutputName() {
 
 //////////////////////////////////////////////////////////////////////////////
 
+static std::vector<std::string> getDefaultLibNames() {
+  std::vector<std::string> result;
+
+  if (noDefaultLib) {
+    deprecation(Loc(), "-nodefaultlib is deprecated, as -defaultlib now "
+                       "overrides the existing list instead of appending to "
+                       "it. Please use the latter instead.");
+  } else if (!global.params.betterC) {
+    const bool addDebugSuffix =
+        (linkDefaultLibDebug && debugLib.getNumOccurrences() == 0);
+    const bool addSharedSuffix = linkAgainstSharedDefaultLibs();
+
+    // Parse comma-separated default library list.
+    std::stringstream libNames(
+        linkDefaultLibDebug && !addDebugSuffix ? debugLib : defaultLib);
+    while (libNames.good()) {
+      std::string lib;
+      std::getline(libNames, lib, ',');
+      if (lib.empty()) {
+        continue;
+      }
+
+      result.push_back((llvm::Twine(lib) + (addDebugSuffix ? "-debug" : "") +
+                        (addSharedSuffix ? "-shared" : ""))
+                           .str());
+    }
+  }
+
+  return result;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
+bool useInternalLLDForLinking() { return linkInternally; }
+
+cl::boolOrDefault linkFullyStatic() { return staticFlag; }
+
+bool linkAgainstSharedDefaultLibs() {
+  // -static enforces static default libs.
+  // Default to shared default libs for DLLs.
+  return staticFlag != cl::BOU_TRUE &&
+         (linkDefaultLibShared == cl::BOU_TRUE ||
+          (linkDefaultLibShared == cl::BOU_UNSET && global.params.dll));
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
 /// Insert an LLVM bitcode file into the module
 static void insertBitcodeIntoModule(const char *bcFile, llvm::Module &M,
                                     llvm::LLVMContext &Context) {
@@ -133,11 +217,13 @@ int linkObjToBinary() {
 
   createDirectoryForFileOrFail(gExePath);
 
+  const auto defaultLibNames = getDefaultLibNames();
+
   if (global.params.targetTriple->isWindowsMSVCEnvironment()) {
-    return linkObjToBinaryMSVC(gExePath, useInternalLinker, staticFlag);
+    return linkObjToBinaryMSVC(gExePath, defaultLibNames);
   }
 
-  return linkObjToBinaryGcc(gExePath, useInternalLinker, staticFlag);
+  return linkObjToBinaryGcc(gExePath, defaultLibNames);
 }
 
 //////////////////////////////////////////////////////////////////////////////
