@@ -130,7 +130,9 @@ DValue *DtoNewClass(Loc &loc, TypeClass *tc, NewExp *newexp) {
     assert(newexp->arguments != NULL);
     DtoResolveFunction(newexp->member);
     DFuncValue dfn(newexp->member, DtoCallee(newexp->member), mem);
-    return DtoCallFunction(newexp->loc, tc, &dfn, newexp->arguments);
+    // ignore ctor return value (C++ ctors on Posix may not return `this`)
+    DtoCallFunction(newexp->loc, tc, &dfn, newexp->arguments);
+    return new DImValue(tc, mem);
   }
 
   assert(newexp->argprefix == NULL);
@@ -536,12 +538,12 @@ static LLConstant *build_class_dtor(ClassDeclaration *cd) {
       DtoCallee(dtor), getPtrToType(LLType::getInt8Ty(gIR->context())));
 }
 
-static ClassFlags::Type build_classinfo_flags(ClassDeclaration *cd) {
+static unsigned build_classinfo_flags(ClassDeclaration *cd) {
   // adapted from original dmd code:
   // toobj.c: ToObjFile::visit(ClassDeclaration*) and
   // ToObjFile::visit(InterfaceDeclaration*)
 
-  ClassFlags::Type flags = ClassFlags::hasOffTi | ClassFlags::hasTypeInfo;
+  auto flags = ClassFlags::hasOffTi | ClassFlags::hasTypeInfo;
   if (cd->isInterfaceDeclaration()) {
     if (cd->isCOMinterface()) {
       flags |= ClassFlags::isCOMclass;
@@ -674,7 +676,7 @@ LLConstant *DtoDefineClassInfo(ClassDeclaration *cd) {
   b.push_funcptr(cd->inv, invVar->type);
 
   // flags
-  ClassFlags::Type flags = build_classinfo_flags(cd);
+  const unsigned flags = build_classinfo_flags(cd);
   b.push_uint(flags);
 
   // deallocator
