@@ -13,7 +13,9 @@
 #include "gen/llvm.h"
 #include "gen/llvmhelpers.h"
 #include "gen/logger.h"
+#include "gen/optimizer.h"
 #include "gen/tollvm.h"
+#include "llvm/IR/MDBuilder.h"
 
 namespace {
 bool isDefinedInFuncEntryBB(LLValue *v) {
@@ -62,7 +64,10 @@ DRValue::DRValue(Type *t, LLValue *v) : DValue(t, v) {
 
 DImValue::DImValue(Type *t, llvm::Value *v) : DRValue(t, v) {
   // TODO: get rid of Tfunction exception
-  assert(t->toBasetype()->ty == Tfunction || v->getType() == DtoType(t));
+  // v may be an addrspace qualified pointer so strip it before doing a pointer
+  // equality check.
+  assert(t->toBasetype()->ty == Tfunction ||
+         stripAddrSpaces(v->getType()) == DtoType(t));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -75,7 +80,9 @@ DConstValue::DConstValue(Type *t, LLConstant *con) : DRValue(t, con) {
 
 DSliceValue::DSliceValue(Type *t, LLValue *pair) : DRValue(t, pair) {
   assert(t->toBasetype()->ty == Tarray);
-  assert(pair->getType() == DtoType(t));
+  // v may be an addrspace qualified pointer so strip it before doing a pointer
+  // equality check.
+  assert(stripAddrSpaces(pair->getType()) == DtoType(t));
 }
 
 DSliceValue::DSliceValue(Type *t, LLValue *length, LLValue *ptr)
@@ -101,7 +108,10 @@ bool DFuncValue::definedInFuncEntryBB() {
 ////////////////////////////////////////////////////////////////////////////////
 
 DLValue::DLValue(Type *t, LLValue *v) : DValue(t, v) {
-  assert(t->toBasetype()->ty == Ttuple || v->getType() == DtoPtrToType(t));
+  // v may be an addrspace qualified pointer so strip it before doing a pointer
+  // equality check.
+  assert(t->toBasetype()->ty == Ttuple ||
+         stripAddrSpaces(v->getType()) == DtoPtrToType(t));
 }
 
 DRValue *DLValue::getRVal() {
@@ -113,6 +123,16 @@ DRValue *DLValue::getRVal() {
   LLValue *rval = DtoLoad(val);
   if (type->toBasetype()->ty == Tbool) {
     assert(rval->getType() == llvm::Type::getInt8Ty(gIR->context()));
+
+    if (isOptimizationEnabled()) {
+      // attach range metadata for i8 being loaded: [0, 2)
+      llvm::MDBuilder mdBuilder(gIR->context());
+      llvm::cast<llvm::LoadInst>(rval)->setMetadata(
+          llvm::LLVMContext::MD_range,
+          mdBuilder.createRange(llvm::APInt(8, 0), llvm::APInt(8, 2)));
+    }
+
+    // truncate to i1
     rval = gIR->ir->CreateTrunc(rval, llvm::Type::getInt1Ty(gIR->context()));
   }
 
