@@ -129,7 +129,7 @@ static void addExplicitArguments(std::vector<LLValue *> &args, AttrSet &attrs,
   std::vector<IrFuncTyArg *> optionalIrArgs;
   for (size_t i = formalDArgCount; i < explicitDArgCount; i++) {
     Type *argType = argexps[i]->type;
-    bool passByVal = gABI->passByVal(argType);
+    bool passByVal = gABI->passByVal(irFty.type, argType);
 
     AttrBuilder initialAttrs;
     if (passByVal) {
@@ -174,13 +174,9 @@ static void addExplicitArguments(std::vector<LLValue *> &args, AttrSet &attrs,
     // evaluate argument expression
     DValue *const dval = DtoArgument(formalParam, argexp);
 
-    // check whether it is an lvalue which might be modified by later argument
-    // expressions
-    const bool isModifiableLvalue =
-        argexp->isLvalue() && dArgIndex != explicitDArgCount - 1;
-
     // load from lvalue/let TargetABI rewrite it/...
-    llvm::Value *llVal = irFty.putParam(*irArg, dval, isModifiableLvalue);
+    llvm::Value *llVal = irFty.putArg(*irArg, dval, argexp->isLvalue(),
+                                      dArgIndex == explicitDArgCount - 1);
 
     const size_t llArgIdx =
         implicitLLArgCount +
@@ -238,7 +234,7 @@ static LLValue *getTypeinfoArrayArgumentForDVarArg(Expressions *argexps,
   const size_t numVariadicArgs = numArgExps - begin;
 
   // build type info array
-  LLType *typeinfotype = DtoType(Type::dtypeinfo->type);
+  LLType *typeinfotype = DtoType(getTypeInfoType());
   LLArrayType *typeinfoarraytype =
       LLArrayType::get(typeinfotype, numVariadicArgs);
 
@@ -261,7 +257,7 @@ static LLValue *getTypeinfoArrayArgumentForDVarArg(Expressions *argexps,
   LLConstant *pinits[] = {
       DtoConstSize_t(numVariadicArgs),
       llvm::ConstantExpr::getBitCast(typeinfomem, getPtrToType(typeinfotype))};
-  LLType *tiarrty = DtoType(Type::dtypeinfo->type->arrayOf());
+  LLType *tiarrty = DtoType(getTypeInfoType()->arrayOf());
   tiinits = LLConstantStruct::get(isaStruct(tiarrty),
                                   llvm::ArrayRef<LLConstant *>(pinits));
   LLValue *typeinfoarrayparam = new llvm::GlobalVariable(
