@@ -817,17 +817,21 @@ public:
     }
     assert(isaPointer(baseValue));
 
-    llvm::Value *offsetValue;
+    llvm::Value *offsetValue = nullptr;
 
     if (e->offset == 0) {
       offsetValue = baseValue;
     } else {
-      uint64_t elemSize = gDataLayout->getTypeAllocSize(
-          baseValue->getType()->getContainedType(0));
-      if (e->offset % elemSize == 0) {
-        // We can turn this into a "nice" GEP.
-        offsetValue = DtoGEPi1(baseValue, e->offset / elemSize);
-      } else {
+      LLType *elemType = baseValue->getType()->getContainedType(0);
+      if (elemType->isSized()) {
+        uint64_t elemSize = gDataLayout->getTypeAllocSize(elemType);
+        if (e->offset % elemSize == 0) {
+          // We can turn this into a "nice" GEP.
+          offsetValue = DtoGEPi1(baseValue, e->offset / elemSize);
+        }
+      }
+
+      if (!offsetValue) {
         // Offset isn't a multiple of base type size, just cast to i8* and
         // apply the byte offset.
         offsetValue =
@@ -1538,7 +1542,7 @@ public:
       if (!e->arguments || e->arguments->dim == 0) {
         IF_LOG Logger::println("default initializer\n");
         // static arrays never appear here, so using the defaultInit is ok!
-        exp = e->newtype->defaultInit(e->loc);
+        exp = defaultInit(e->newtype, e->loc);
       } else {
         IF_LOG Logger::println("uniform constructor\n");
         assert(e->arguments->dim == 1);
