@@ -41,15 +41,11 @@ bool DtoIsInMemoryOnly(Type *type) {
   return (t == Tstruct || t == Tsarray);
 }
 
-RET retStyle(TypeFunction *tf) {
-  bool sret = gABI->returnInArg(tf);
-  return sret ? RETstack : RETregs;
-}
-
 bool DtoIsReturnInArg(CallExp *ce) {
-  TypeFunction *tf = static_cast<TypeFunction *>(ce->e1->type->toBasetype());
-  if (tf->ty == Tfunction && (!ce->f || !DtoIsIntrinsic(ce->f))) {
-    return retStyle(tf) == RETstack;
+  Type *t = ce->e1->type->toBasetype();
+  if (t->ty == Tfunction && (!ce->f || !DtoIsIntrinsic(ce->f))) {
+    return gABI->returnInArg(static_cast<TypeFunction *>(t),
+                             ce->f && ce->f->needThis());
   }
   return false;
 }
@@ -234,7 +230,15 @@ LinkageWithCOMDAT DtoLinkage(Dsymbol *sym) {
 }
 
 bool supportsCOMDAT() {
-  return !global.params.targetTriple->isOSBinFormatMachO();
+  const auto &triple = *global.params.targetTriple;
+  return !(triple.isOSBinFormatMachO() ||
+#if LDC_LLVM_VER >= 500
+           triple.isOSBinFormatWasm()
+#else
+           triple.getArch() == llvm::Triple::wasm32 ||
+           triple.getArch() == llvm::Triple::wasm64
+#endif
+  );
 }
 
 void setLinkage(LinkageWithCOMDAT lwc, llvm::GlobalObject *obj) {
@@ -658,8 +662,7 @@ LLStructType *DtoModuleReferenceType() {
   LLStructType *st = LLStructType::create(gIR->context(), "ModuleReference");
 
   // add members
-  LLType *types[] = {getPtrToType(st),
-                     DtoType(Module::moduleinfo->type->pointerTo())};
+  LLType *types[] = {getPtrToType(st), DtoPtrToType(getModuleInfoType())};
 
   // resolve type
   st->setBody(types);
