@@ -51,9 +51,8 @@ LLGlobalVariable *IrAggr::getVtblSymbol() {
 
   LLType *vtblTy = stripModifiers(type)->ctype->isClass()->getVtblType();
 
-  vtbl =
-      getOrCreateGlobal(aggrdecl->loc, gIR->module, vtblTy, true,
-                        llvm::GlobalValue::ExternalLinkage, nullptr, irMangle);
+  vtbl = declareGlobal(aggrdecl->loc, gIR->module, vtblTy, irMangle,
+                       /*isConstant=*/true);
 
   return vtbl;
 }
@@ -71,15 +70,14 @@ LLGlobalVariable *IrAggr::getClassInfoSymbol() {
   // The type is also ClassInfo for interfaces – the actual TypeInfo for them
   // is a TypeInfo_Interface instance that references __ClassZ in its "base"
   // member.
-  ClassDeclaration *cinfo = Type::typeinfoclass;
-  DtoType(cinfo->type);
-  IrTypeClass *tc = stripModifiers(cinfo->type)->ctype->isClass();
+  Type *cinfoType = getClassInfoType();
+  DtoType(cinfoType);
+  IrTypeClass *tc = stripModifiers(cinfoType)->ctype->isClass();
   assert(tc && "invalid ClassInfo type");
 
   // classinfos cannot be constants since they're used as locks for synchronized
-  classInfo = getOrCreateGlobal(
-      aggrdecl->loc, gIR->module, tc->getMemoryLLType(), false,
-      llvm::GlobalValue::ExternalLinkage, nullptr, irMangle);
+  classInfo = declareGlobal(aggrdecl->loc, gIR->module, tc->getMemoryLLType(),
+                            irMangle, false);
 
   // Generate some metadata on this ClassInfo if it's for a class.
   ClassDeclaration *classdecl = aggrdecl->isClassDeclaration();
@@ -116,6 +114,11 @@ LLGlobalVariable *IrAggr::getClassInfoSymbol() {
 
 //////////////////////////////////////////////////////////////////////////////
 
+static Type *getInterfacesArrayType() {
+  getClassInfoType(); // check declaration in object.d
+  return Type::typeinfoclass->fields[3]->type;
+}
+
 LLGlobalVariable *IrAggr::getInterfaceArraySymbol() {
   if (classInterfacesArray) {
     return classInterfacesArray;
@@ -127,18 +130,15 @@ LLGlobalVariable *IrAggr::getInterfaceArraySymbol() {
   assert(n > 0 && "getting ClassInfo.interfaces storage symbol, but we "
                   "don't implement any interfaces");
 
-  LLType *InterfaceTy = DtoType(Type::typeinfoclass->fields[3]->type->nextOf());
+  LLType *InterfaceTy = DtoType(getInterfacesArrayType()->nextOf());
 
   // create Interface[N]
   const auto irMangle = getIRMangledInterfaceInfosSymbolName(cd);
 
   LLArrayType *array_type = llvm::ArrayType::get(InterfaceTy, n);
 
-  // We keep the global as external for now and only consider template linkage
-  // if we emit the initializer later.
-  classInterfacesArray =
-      getOrCreateGlobal(cd->loc, gIR->module, array_type, true,
-                        llvm::GlobalValue::ExternalLinkage, nullptr, irMangle);
+  classInterfacesArray = declareGlobal(cd->loc, gIR->module, array_type,
+                                       irMangle, /*isConstant=*/true);
 
   return classInterfacesArray;
 }
@@ -164,8 +164,13 @@ LLConstant *IrAggr::getVtblInit() {
   // start with the classinfo
   llvm::Constant *c;
   if (!cd->isCPPclass()) {
-    c = getClassInfoSymbol();
-    c = DtoBitCast(c, voidPtrType);
+    if (global.params.useTypeInfo && Type::dtypeinfo) {
+      c = getClassInfoSymbol();
+      c = DtoBitCast(c, voidPtrType);
+    } else {
+      // use null if there are no TypeInfos
+      c = llvm::Constant::getNullValue(voidPtrType);
+    }
     constants.push_back(c);
   }
 
@@ -252,15 +257,48 @@ LLConstant *IrAggr::getClassInfoInit() {
 
 //////////////////////////////////////////////////////////////////////////////
 
-llvm::GlobalVariable *IrAggr::getInterfaceVtbl(BaseClass *b, bool new_instance,
-                                               size_t interfaces_index) {
+llvm::GlobalVariable *IrAggr::getInterfaceVtblSymbol(BaseClass *b,
+                                                     size_t interfaces_index) {
   auto it = interfaceVtblMap.find({b->sym, interfaces_index});
   if (it != interfaceVtblMap.end()) {
     return it->second;
   }
 
+  ClassDeclaration *cd = aggrdecl->isClassDeclaration();
+  assert(cd && "not a class aggregate");
+
+  llvm::Type *vtblType = LLArrayType::get(getVoidPtrType(), b->sym->vtbl.dim);
+
+  // Thunk prefix
+  char thunkPrefix[16];
+  int thunkLen = sprintf(thunkPrefix, "Thn%d_", b->offset);
+  char thunkPrefixLen[16];
+  sprintf(thunkPrefixLen, "%d", thunkLen);
+
+  OutBuffer mangledName;
+  mangledName.writestring("_D");
+  mangleToBuffer(cd, &mangledName);
+  mangledName.writestring("11__interface");
+  mangleToBuffer(b->sym, &mangledName);
+  mangledName.writestring(thunkPrefixLen);
+  mangledName.writestring(thunkPrefix);
+  mangledName.writestring("6__vtblZ");
+
+  const auto irMangle = getIRMangledVarName(mangledName.peekString(), LINKd);
+
+  LLGlobalVariable *gvar =
+      declareGlobal(cd->loc, gIR->module, vtblType, irMangle, /*isConstant=*/true);
+
+  // insert into the vtbl map
+  interfaceVtblMap.insert({{b->sym, interfaces_index}, gvar});
+
+  return gvar;
+}
+
+void IrAggr::defineInterfaceVtbl(BaseClass *b, bool new_instance,
+                                 size_t interfaces_index) {
   IF_LOG Logger::println(
-      "Building vtbl for implementation of interface %s in class %s",
+      "Defining vtbl for implementation of interface %s in class %s",
       b->sym->toPrettyChars(), aggrdecl->toPrettyChars());
   LOG_SCOPE;
 
@@ -273,32 +311,34 @@ llvm::GlobalVariable *IrAggr::getInterfaceVtbl(BaseClass *b, bool new_instance,
   std::vector<llvm::Constant *> constants;
   constants.reserve(vtbl_array.dim);
 
+  char thunkPrefix[16];
+  sprintf(thunkPrefix, "Thn%d_", b->offset);
+
   const auto voidPtrTy = getVoidPtrType();
 
   if (!b->sym->isCPPinterface()) { // skip interface info for CPP interfaces
-    // index into the interfaces array
-    llvm::Constant *idxs[2] = {DtoConstSize_t(0),
-                               DtoConstSize_t(interfaces_index)};
+    if (global.params.useTypeInfo && Type::dtypeinfo) {
+      // index into the interfaces array
+      llvm::Constant *idxs[2] = {DtoConstSize_t(0),
+                                 DtoConstSize_t(interfaces_index)};
 
-    llvm::GlobalVariable *interfaceInfosZ = getInterfaceArraySymbol();
-    llvm::Constant *c = llvm::ConstantExpr::getGetElementPtr(
-        isaPointer(interfaceInfosZ)->getElementType(),
-        interfaceInfosZ, idxs, true);
+      llvm::GlobalVariable *interfaceInfosZ = getInterfaceArraySymbol();
+      llvm::Constant *c = llvm::ConstantExpr::getGetElementPtr(
+          isaPointer(interfaceInfosZ)->getElementType(), interfaceInfosZ, idxs,
+          true);
 
-    constants.push_back(DtoBitCast(c, voidPtrTy));
+      constants.push_back(DtoBitCast(c, voidPtrTy));
+    } else {
+      // use null if there are no TypeInfos
+      constants.push_back(llvm::Constant::getNullValue(voidPtrTy));
+    }
   }
 
-  // Thunk prefix
-  char thunkPrefix[16];
-  int thunkLen = sprintf(thunkPrefix, "Thn%d_", b->offset);
-  char thunkPrefixLen[16];
-  sprintf(thunkPrefixLen, "%d", thunkLen);
-
   // add virtual function pointers
   size_t n = vtbl_array.dim;
   for (size_t i = b->sym->vtblOffset(); i < n; i++) {
-    Dsymbol *dsym = static_cast<Dsymbol *>(vtbl_array.data[i]);
-    if (dsym == nullptr) {
+    FuncDeclaration *fd = vtbl_array[i];
+    if (!fd) {
       // FIXME
       // why is this null?
       // happens for mini/s.d
@@ -306,9 +346,6 @@ llvm::GlobalVariable *IrAggr::getInterfaceVtbl(BaseClass *b, bool new_instance,
       continue;
     }
 
-    FuncDeclaration *fd = dsym->isFuncDeclaration();
-    assert(fd && "vtbl entry not a function");
-
     assert((!fd->isAbstract() || fd->fbody) &&
            "null symbol in interface implementation vtable");
 
@@ -415,7 +452,8 @@ llvm::GlobalVariable *IrAggr::getInterfaceVtbl(BaseClass *b, bool new_instance,
       // call the real vtbl function.
       llvm::CallInst *call = gIR->ir->CreateCall(callee, args);
       call->setCallingConv(irFunc->getCallingConv());
-      call->setTailCallKind(llvm::CallInst::TCK_Tail);
+      call->setTailCallKind(thunk->isVarArg() ? llvm::CallInst::TCK_MustTail
+                                              : llvm::CallInst::TCK_Tail);
 
       // return from the thunk
       if (thunk->getReturnType() == LLType::getVoidTy(gIR->context())) {
@@ -439,27 +477,24 @@ llvm::GlobalVariable *IrAggr::getInterfaceVtbl(BaseClass *b, bool new_instance,
   llvm::Constant *vtbl_constant = LLConstantArray::get(
       LLArrayType::get(voidPtrTy, constants.size()), constants);
 
-  OutBuffer mangledName;
-  mangledName.writestring("_D");
-  mangleToBuffer(cd, &mangledName);
-  mangledName.writestring("11__interface");
-  mangleToBuffer(b->sym, &mangledName);
-  mangledName.writestring(thunkPrefixLen);
-  mangledName.writestring(thunkPrefix);
-  mangledName.writestring("6__vtblZ");
+  // define the global
+  const auto gvar = getInterfaceVtblSymbol(b, interfaces_index);
+  defineGlobal(gvar, vtbl_constant, cd);
+}
 
-  const auto irMangle = getIRMangledVarName(mangledName.peekString(), LINKd);
+void IrAggr::defineInterfaceVtbls() {
+  const size_t n = interfacesWithVtbls.size();
+  assert(n == stripModifiers(type)->ctype->isClass()->getNumInterfaceVtbls() &&
+         "inconsistent number of interface vtables in this class");
 
-  const auto lwc = DtoLinkage(cd);
-  LLGlobalVariable *GV =
-      getOrCreateGlobal(cd->loc, gIR->module, vtbl_constant->getType(), true,
-                        lwc.first, vtbl_constant, irMangle);
-  setLinkage(lwc, GV);
+  for (size_t i = 0; i < n; ++i) {
+    auto baseClass = interfacesWithVtbls[i];
 
-  // insert into the vtbl map
-  interfaceVtblMap.insert({{b->sym, interfaces_index}, GV});
+    // false when it's not okay to use functions from super classes
+    bool newinsts = (baseClass->sym == aggrdecl->isClassDeclaration());
 
-  return GV;
+    defineInterfaceVtbl(baseClass, newinsts, i);
+  }
 }
 
 bool IrAggr::isPacked() const {
@@ -479,10 +514,10 @@ LLConstant *IrAggr::getClassInfoInterfaces() {
   assert(stripModifiers(type)->ctype->isClass()->getNumInterfaceVtbls() == n &&
          "inconsistent number of interface vtables in this class");
 
-  VarDeclaration *interfaces_idx = Type::typeinfoclass->fields[3];
+  Type *interfacesArrayType = getInterfacesArrayType();
 
   if (n == 0) {
-    return getNullValue(DtoType(interfaces_idx->type));
+    return getNullValue(DtoType(interfacesArrayType));
   }
 
   // Build array of:
@@ -497,10 +532,10 @@ LLConstant *IrAggr::getClassInfoInterfaces() {
   LLSmallVector<LLConstant *, 6> constants;
   constants.reserve(cd->vtblInterfaces->dim);
 
-  LLType *classinfo_type = DtoType(Type::typeinfoclass->type);
+  LLType *classinfo_type = DtoType(getClassInfoType());
   LLType *voidptrptr_type = DtoType(Type::tvoid->pointerTo()->pointerTo());
-  VarDeclaration *idx = Type::typeinfoclass->fields[3];
-  LLStructType *interface_type = isaStruct(DtoType(idx->type->nextOf()));
+  LLStructType *interface_type =
+      isaStruct(DtoType(interfacesArrayType->nextOf()));
   assert(interface_type);
 
   for (size_t i = 0; i < n; ++i) {
@@ -547,12 +582,11 @@ LLConstant *IrAggr::getClassInfoInterfaces() {
   // create and apply initializer
   LLConstant *arr = LLConstantArray::get(array_type, constants);
   auto ciarr = getInterfaceArraySymbol();
-  ciarr->setInitializer(arr);
-  setLinkage(cd, ciarr);
+  defineGlobal(ciarr, arr, cd);
 
   // return null, only baseclass provide interfaces
   if (cd->vtblInterfaces->dim == 0) {
-    return getNullValue(DtoType(interfaces_idx->type));
+    return getNullValue(DtoType(interfacesArrayType));
   }
 
   // only the interface explicitly implemented by this class
