@@ -11,6 +11,7 @@
 #include "module.h"
 #include "gen/llvm.h"
 #include "gen/irstate.h"
+#include "gen/llvmhelpers.h"
 #include "gen/mangling.h"
 #include "gen/tollvm.h"
 #include "ir/irdsymbol.h"
@@ -25,9 +26,9 @@ llvm::GlobalVariable *IrModule::moduleInfoSymbol() {
 
   const auto irMangle = getIRMangledModuleInfoSymbolName(M);
 
-  moduleInfoVar = new llvm::GlobalVariable(
-      gIR->module, llvm::StructType::create(gIR->context()), false,
-      llvm::GlobalValue::ExternalLinkage, nullptr, irMangle);
+  moduleInfoVar =
+      declareGlobal(Loc(), gIR->module,
+                    llvm::StructType::create(gIR->context()), irMangle, false);
   return moduleInfoVar;
 }
 
